//----------------------------------------------------------------------
//This code is developed as part of the Java CoG Kit project
//The terms of the license can be found at http://www.cogkit.org/license
//This message may not be removed or altered.
//----------------------------------------------------------------------

/*
 * Created on Oct 11, 2005
 */
package org.globus.cog.abstraction.impl.scheduler.sge;

import java.io.BufferedReader;
import java.io.CharArrayReader;
import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.globus.cog.abstraction.impl.common.execution.WallTime;
import org.globus.cog.abstraction.impl.scheduler.common.AbstractExecutor;
import org.globus.cog.abstraction.impl.scheduler.common.AbstractProperties;
import org.globus.cog.abstraction.impl.scheduler.common.AbstractQueuePoller;
import org.globus.cog.abstraction.impl.scheduler.common.Job;
import org.globus.cog.abstraction.impl.scheduler.common.ProcessListener;
import org.globus.cog.abstraction.interfaces.FileLocation;
import org.globus.cog.abstraction.interfaces.JobSpecification;
import org.globus.cog.abstraction.interfaces.Task;

public class SGEExecutor extends AbstractExecutor {
    public static final Logger logger = Logger.getLogger(SGEExecutor.class);

    public SGEExecutor(Task task, ProcessListener listener) {
        super(task, listener);
    }

    protected void writeAttr(String attrName, String arg, Writer wr,
            String defaultValue) throws IOException {
        Object value = getSpec().getAttribute(attrName);
        if (value != null) {
            wr.write("#$ " + arg + String.valueOf(value) + '\n');
        }
        else if (defaultValue != null) {
            wr.write("#$ " + arg + String.valueOf(defaultValue) + '\n');
        }
    }

    protected void writeAttr(String attrName, String arg, Writer wr)
            throws IOException {
        writeAttr(attrName, arg, wr, null);
    }

    protected void writeAttrValue(String value, String arg, Writer wr ) throws IOException {
        wr.write("#$ " + arg + value + '\n');
    }

    protected void writeWallTime(Writer wr) throws IOException {
        Object walltime = getSpec().getAttribute("maxwalltime");
        if (walltime != null) {
            wr.write("#$ -l h_rt="
                    + WallTime.normalize(walltime.toString(), "sge-native")
                    + '\n');
        }
    }

    protected void writeScript(Writer wr, String exitcodefile, String stdout,
            String stderr) throws IOException {
        Task task = getTask();
        JobSpecification spec = getSpec();
        String type = (String) spec.getAttribute("jobType");
        boolean multiple = false;
        if ("multiple".equals(type)) {
            multiple = true;
        }

        wr.write("#!/bin/bash\n");
        wr.write("#$ -N " + task.getName() + '\n');
        // ranger requires this. might as well be default
        wr.write("#$ -V\n");
        writeAttr("project", "-A ", wr);

// FIXME: testing this change: MW

        Object countValue = getSpec().getAttribute("count");
	int count;

        if (countValue != null)
            count = Integer.valueOf(String.valueOf(countValue)).intValue();
	else 
	    count = 1;

        // FIXME: wpn is only meaningful for coasters; is 1 ok otherwise?
	// should we flag wpn as error if not coasters?

	Object wpnValue = getAttribute(spec, "workerspernode", "1");
	int wpn = Integer.valueOf(String.valueOf(wpnValue)).intValue();
	logger.info("FETCH OF WPN: " + wpn); // FIXME: DB

	count *= wpn;
	logger.info("FETCH OF PE: " + getAttribute(spec, "pe", "NO pe"));
	logger.info("FETCH OF CPN: " + getAttribute(spec, "corespernode", "NO cpn"));
        writeAttrValue(String.valueOf(count), "-pe "
                + getAttribute(spec, "pe", getSGEProperties().getDefaultPE())
                + " ", wr);

// FIXME: END OF MW CHANGE

        writeWallTime(wr);
        writeAttr("queue", "-q ", wr);
        if (spec.getStdInput() != null) {
            wr.write("#$ -i " + quote(spec.getStdInput()) + '\n');
        }
        wr.write("#$ -o " + quote(stdout) + '\n');
        wr.write("#$ -e " + quote(stderr) + '\n');

        if (!spec.getEnvironmentVariableNames().isEmpty()) {
            wr.write("#$ -v ");
            Iterator i = spec.getEnvironmentVariableNames().iterator();
            while (i.hasNext()) {
                String name = (String) i.next();
                wr.write(name);
                wr.write('=');
                wr.write(quote(spec.getEnvironmentVariable(name)));
                if (i.hasNext()) {
                    wr.write(',');
                }
            }
            wr.write('\n');
        }

        if (logger.isDebugEnabled()) {
            logger.debug("Job type: " + type);
        }
        if (type != null) {
            String wrapper = Properties.getProperties().getProperty(
                "wrapper." + type);
            if (logger.isDebugEnabled()) {
                logger.debug("Wrapper: " + wrapper);
            }
            if (wrapper != null) {
                wrapper = replaceVars(wrapper);
                wr.write(wrapper);
                wr.write(' ');
            }
            if (logger.isDebugEnabled()) {
                logger.debug("Wrapper after variable substitution: " + wrapper);
            }
        }
        if (spec.getDirectory() != null) {
            wr.write("cd " + quote(spec.getDirectory()) + " && ");
        }

        if (multiple) {
            writeMultiJobPreamble(wr, exitcodefile);
        }
        wr.write(quote(spec.getExecutable()));
        List args = spec.getArgumentsAsList();
        if (args != null && args.size() > 0) {
            wr.write(' ');
            Iterator i = args.iterator();
            while (i.hasNext()) {
                wr.write(quote((String) i.next()));
                if (i.hasNext()) {
                    wr.write(' ');
                }
            }
        }

        if (spec.getStdInput() != null) {
            wr.write(" < " + quote(spec.getStdInput()));
        }
        if (multiple) {
            writeMultiJobPostamble(wr);
        }
        else {
            wr.write('\n');
            wr.write("/bin/echo $? >" + exitcodefile + '\n');
        }
        wr.close();
    }
    
    protected void writeMultiJobPreamble(Writer wr, String exitcodefile)
            throws IOException {
// FIXME:MW        wr.write("NODES=`cat $PE_HOSTFILE | awk '{ for(i=0;i<$2;i++){print $1} }'`\n");
        wr.write("NODES=`cat $PE_HOSTFILE | awk '{ print $1 }'`\n");
        wr.write("ECF=" + exitcodefile + "\n");
        wr.write("INDEX=0\n");
        wr.write("for NODE in $NODES; do\n");
        wr.write("  echo \"N\" >$ECF.$INDEX\n");
        wr.write("  ssh $NODE /bin/bash -c \\\" \"");
    }

    private String getAttribute(JobSpecification spec, String name,
            String defaultValue) {
        Object value = spec.getAttribute(name);
        if (value == null) {
            return defaultValue;
        }
        else {
            return value.toString();
        }
    }

    protected String getName() {
        return "SGE";
    }

    protected AbstractProperties getProperties() {
        return Properties.getProperties();
    }
    
    protected Properties getSGEProperties() {
        return (Properties) getProperties();
    }
    
     public static final Pattern JOB_ID_LINE = Pattern.compile(".*[Yy]our job (\\d+) \\(.*\\) has been submitted");
    // public static final Pattern JOB_ID_LINE = Pattern.compile("[Yy]our job (\\d+) \\(.*\\) has been submitted");
    // public static final Pattern JOB_ID_LINE = Pattern.compile("[Yy]our job \\([0-9]\\+\\) .* has been submitted");

    protected String parseSubmitCommandOutput(String out) throws IOException {
        // > your job 2494189 ("t1.sub") has been submitted
        BufferedReader br = new BufferedReader(new CharArrayReader(out.toCharArray()));
        String line = br.readLine();
	if (logger.isInfoEnabled()) {
	    logger.info("parseSubmitCommandOutput: out=" + out);
	}
        while (line != null) {
	    if (logger.isInfoEnabled()) {
		logger.info("parseSubmitCommandOutput: line=" + line);
	    }
            Matcher m = JOB_ID_LINE.matcher(line);
            if (m.matches()) {
                String id = m.group(1);
                if (logger.isInfoEnabled()) {
                    logger.info("Job id from qsub: " + id);
                }
                return id;
            }
            line = br.readLine();
        }
        throw new IOException("None of the qsub lines matches the required patten: " + JOB_ID_LINE);
    }

    private static final String[] QSUB_PARAMS = new String[] {};

    protected String[] getAdditionalSubmitParameters() {
        return QSUB_PARAMS;
    }

    protected Job createJob(String jobid, String stdout,
            FileLocation stdOutputLocation, String stderr,
            FileLocation stdErrorLocation, String exitcode,
            AbstractExecutor executor) {
        return new Job(jobid, stdout, stdOutputLocation, stderr,
            stdErrorLocation, exitcode, executor);
    }

    private static QueuePoller poller;

    protected AbstractQueuePoller getQueuePoller() {
        synchronized (SGEExecutor.class) {
            if (poller == null) {
                poller = new QueuePoller(getProperties());
                poller.start();
            }
            return poller;
        }
    }
}
